package Minas::BigMac;

########################################################################
# PACKAGE: Minas::BigMac
#
# Copyright (C), 2002
#
# Bridget Thomson McInnes, bthomson@d.umn.edu
# Deodatta Y Bhoite,       bhoi0001@d.umn.edu
# Kailash Aurangabadkar,   aura0011@d.umn.edu
# Nitin Agarwal,           agar0054@d.umn.edu
# Yanhua Li,               lixx0333@d.umn.edu
#
# University of Minnesota, Duluth
#
# USAGE:
#           use Minas::BigMac
#
# The package BigMac allows a user to access different aspects of 
# the MacQuarie's machine readable dictionary. For more information 
# please view the perl doc
#
#           perldoc Minas::BigMac
#
#######################################################################

use NDBM_File;
use Fcntl;
require Exporter;

our @ISA = qw(Exporter);
our @EXPORT = qw (new,setDictionaryLocation,setThesaurusLocation,getPronunciations,getDefinitions,getIdiomaticPhrases,getEtymology,getSynonyms);
our $VERSION = 1.0;

#Global variables
my $dlocation;
my $tlocation;
my %linenumhash;
my %theshash;
my %wordhash;
my $db;
my $tb;
my @dictlines;
my @theslines;

# Preloaded methods go here.
# method to instantiate the class
sub new{
    my $class = shift;
    my $self = { };
    bless($self, $class);
    return $self;
}

# sets location for the dictionary and indexes it
sub setDictionaryLocation{
    my $class = shift;
    $dlocation = shift;
    my $linenum=0;
    my $line;
    my $hwd;
    my $dtext;
    my @words;
    my $word;
    #
    # Reference: http://www.sunsite.ualberta.ca/Documentation/Misc/perl-5.6.1/pod/perldbmfilter.html
    #
    if(-e "dicthash.pag"){
	$db = tie (%linenumhash,'NDBM_File', "dicthash", O_RDWR|O_CREAT, 0640) or die "Error: Cannot open file for writing\n";
	$db->filter_fetch_key  ( sub { s/\0$//    } ) ;
	$db->filter_store_key  ( sub { $_ .= "\0" } ) ;
	$db->filter_fetch_value( 
       			 sub { no warnings 'uninitialized' ;s/\0$// } ) ;
	$db->filter_store_value( sub { $_ .= "\0" } ) ;
	open(DICT,$dlocation) or die "Error: Cannot open dictionary file:$dlocation\n";
	while($line = <DICT>){
	    chomp($line);
	    $linenum++;
	    push @dictlines,$line;
	    if($line =~m/<RECORD .*?>/){
		do{
		    if($line =~ m/<DTEXT>(.*?)<\/DTEXT>/){
			$line=$';#'
			if(defined $1){
			    $dtext=$1;
			    $dtext=~ s/[^a-zA-Z]/ /g;
			    @words = split(/\s+/,$dtext);
			    foreach $word (@words){
				if(exists $wordhash{$word}){
				    $wordhash{$word}.=":$linenum";
				}else{
				    $wordhash{$word}=$linenum;
				}
			    }
			}else{
			    $dtext="";
			}
		    }
		    else{
			$dtext="";
		    }
		}while($dtext);
	    }
	}
	#print "lines...$#dictlines\n";
	close(DICT);
    }else{
	$db = tie (%linenumhash,'NDBM_File', "dicthash", O_RDWR|O_CREAT, 0640) or die "Error: Cannot open file for writing\n";
	$db->filter_fetch_key  ( sub { s/\0$//    } ) ;
	$db->filter_store_key  ( sub { $_ .= "\0" } ) ;
	$db->filter_fetch_value( 
       			 sub { no warnings 'uninitialized' ;s/\0$// } ) ;
	$db->filter_store_value( sub { $_ .= "\0" } ) ;
	open(DICT,$dlocation) or die "Error: Cannot open dictionary file:$dlocation\n";
	while($line = <DICT>){
	    chomp($line);
	    $linenum++;
	    push @dictlines,$line;
	    if($line =~m/<RECORD .*?>/){
		$line =~ m/<HEAD>(.*?)<\/HEAD>/;
		$hwd=$1;
		#print "$hwd...$linenum\n";
		$linenumhash{$hwd}=$linenum;
		do{
		    if($line =~ m/<DTEXT>(.*?)<\/DTEXT>/){
			$line=$';#'
			if(defined $1){
			    $dtext=$1;
			    $dtext=~ s/[^a-zA-Z]/ /g;
			    @words = split(/\s+/,$dtext);
			    foreach $word (@words){
				if(exists $wordhash{$word}){
				    $wordhash{$word}.=":$linenum";
				}else{
				    $wordhash{$word}=$linenum;
				}
			    }
			}else{
			    $dtext="";
			}
		    }
		    else{
			$dtext="";
		    }
		}while($dtext);
	    }
	}
	close(DICT);
    }
}

# set location for thesaurus file and index it
sub setThesaurusLocation{
    my $class = shift;
    $tlocation = shift;
    my $line;
    my @entries;
    my $linenum=0;
    #
    # Reference: http://www.sunsite.ualberta.ca/Documentation/Misc/perl-5.6.1/pod/perldbmfilter.html
    #
    if(-e "theshash.pag"){
	$tb = tie (%theshash,'NDBM_File', "theshash", O_RDWR|O_CREAT, 0640) or die "Error: Cannot open file for writing\n";
	$tb->filter_fetch_key  ( sub { s/\0$//    } ) ;
	$tb->filter_store_key  ( sub { $_ .= "\0" } ) ;
	$tb->filter_fetch_value( 
       			 sub { no warnings 'uninitialized' ;s/\0$// } ) ;
	$tb->filter_store_value( sub { $_ .= "\0" } ) ;
	open(THES, $tlocation);
	while($line=<THES>){
	    $line =~ s/[\r\n]//g;
	    push @theslines, $line;
	}
	close(THES);
    }else{
	$tb = tie (%theshash,'NDBM_File', "theshash", O_RDWR|O_CREAT, 0640) or die "Error: Cannot open file for writing\n";
	$tb->filter_fetch_key  ( sub { s/\0$//    } ) ;
	$tb->filter_store_key  ( sub { $_ .= "\0" } ) ;
	$tb->filter_fetch_value( 
       			 sub { no warnings 'uninitialized' ;s/\0$// } ) ;
	$tb->filter_store_value( sub { $_ .= "\0" } ) ;
	open(THES,$tlocation) or die "Error: Cannot open dictionary file:$dlocation\n";
	while($line = <THES>){
	    $line =~ s/[\r\n]//g;
	    $linenum++;
	    push @theslines, $line;
	    @entries = split(/\|/,$line);
	    if($theshash{$entries[0]}){
	    }else{
		$theshash{$entries[0]}=$linenum;
	    }
	}
	close(THES);
    }
}

# retrieves record(s) for a given word
sub query{
    my $word = shift;
    my $record="";
    my $num=1;
    my $key = $word."<SUP>$num</SUP>";
    my $flag=1;
    while($linenumhash{$key}){
	$flag=0;
	$record .= "$dictlines[$linenumhash{$key}-1]\n";
	$num++;
	$key = $word."<SUP>$num</SUP>";
    }
    if($flag){
	$record.="$dictlines[$linenumhash{$word}-1]\n";
    }
    return $record;
}

# gets pronunciations for the word
sub getPronunciations{
    my $class = shift;
    my $word = shift;
    my $rec="";
    my $pron="";
    my %pronounciations;
    my $dum;
    $rec = query($word);
    do{
	if($rec =~ m/<PRN TYPE=SAY>(.*?)<\/PRN>/){
	    if(defined $1){
		$pron=$1;
	    }
	    if($pron){
		$pronounciations{$pron}=1;
	    }
	    $rec=$'; #'
	}else{
	    $pron="";
	}
    }while($pron);
    return %pronounciations;
}

# gets definitions for the word
sub getDefinitions{
    my $class = shift;
    my $word = shift;
    my $rec="";
    my $def="";
    my @definitions;
    my $dum;
    $rec = query($word);
    do{
	if($rec =~ m/<DTEXT>(.*?)<\/DTEXT>/){
	    if(defined $1){
		$def=$1;
	    }
	    if($def){
		push @definitions, $def;
	    }
	    $rec=$'; #'
	}else{
	    $def="";
	}
    }while($def);
    return @definitions;
}

# gets idiomatic phrases for the word
sub getIdiomaticPhrases{
    my $class = shift;
    my $word = shift;
    my $rec = query($word);
    my $ip="";
    my @iphrases;
    do{
	if($rec=~ m/<IP>(.*?)<\/IP>/){
	    if(defined $1){
		$ip=$1;
	    }
	    if($ip){
		push @iphrases, $ip;
	    }
	    $rec=$'; #'
	    }else{
		$ip="";
	    }
    }while($ip);
    return @iphrases;
}

# gets etymology for the word
sub getEtymology{
    my $class=shift;
    my $word=shift;
    my $langregex=shift;
    my $rec= query($word);
    my $ety="";
    my $entry="";
    my %etymology;
    my $lang="";
    my $rem;
    if($rec=~ m/<ETY>(.*?)<\/ETY>/){
	if(defined $1){
	    $ety="<ETY>".$1."</ETY>";
	}
    }
    if($ety){
	do{
	    $ety=~ m/<LANG>(.*?)(<LANG>|<\/ETY>)/;
	    if(defined $1){
		$entry = "<LANG>".$1;
	    }else{
		$entry ="";
	    }
	    if($entry){
		$entry=~ m/<LANG>(.*?)<\/LANG>/;
		$rem=$'; #'
		if(defined $1){
		    $lang=$1;
		    if($lang=~ m/$langregex/i){
			$etymology{$lang} = $rem; #'
		    }
		}
	    }
	    if(defined $2){
		if($2 eq "</ETY>"){
		    $entry="";
		}
		$ety=$2.$'; #'
	    }
	}while($entry);
    }
    return %etymology;
}

#### deprecated
# gets synonyms for the word
sub getSynonymsOld{
    my $class=shift;
    my $word=shift;
    my $th="";
    my %t;
    my $rec= query($word);
    my @tr;
    my $key;
    my @output;
    my @temparr;
    my $tp;
    my $flag=1;
    my $l;

    do{
	if($rec =~ m/<THES>(.*?)<\/THES>/){
	    if(defined $1){
		$th=$1;
	    }
	    if($th){
		$th =~ s/\*//g;
		@tr = split(/\./,$th);
		$tr[0] =~ s/^0*//;
		$tr[1] =~ s/^0*//;
		$tr[2] =~ s/^0*//;
		$tr[2] =~ s/0$//;
		$th="$tr[0].$tr[1].$tr[2]";
		$t{$th}=1;
	    }
	    $rec = $'; #'
	}else{
	    $th="";
	}
    }while($th);
    foreach $key (keys %t){
	$flag=1;
	if($theshash{$key}){
	    $linenum = $theshash{$key}-1;
	    while($flag){
		$l = $theslines[$linenum++];
		@temparr = split(/\|/,$l);
		if($temparr[0] eq $key){
		    $temparr[3] =~ s/<SUBPARA HEAD> //;
		    push @output, $temparr[3];
		    $flag=1;
		}else{
		    $flag=0;
		}
	    }
	}
    }
    return @output;
}

# gets synonyms for the word labeled by the thesaurus entry number
sub getSynonyms{
    my $class=shift;
    my $word=shift;
    my $th="";
    my %t;
    my $rec;
    my @tr;
    my $key;
    my @output;
    my @temparr;
    my $tp;
    my $flag=1;
    my $l;
    my %thesoutput;
    $rec = query($word);
    do{
	if($rec =~ m/<THES>(.*?)<\/THES>/){
	    if(defined $1){
		$th=$1;
	    }
	    if($th){
		$th =~ s/\*//g;
		@tr = split(/\./,$th);
		$tr[0] =~ s/^0*//;
		$tr[1] =~ s/^0*//;
		$tr[2] =~ s/^0*//;
		$tr[2] =~ s/0$//;
		$th="$tr[0].$tr[1].$tr[2]";
		$t{$th}=1;
	    }
	    $rec = $'; #'
	}else{
	    $th="";
	}
    }while($th);
    foreach $key (keys %t){
	$flag=1;
	if($theshash{$key}){
	    $linenum = $theshash{$key}-1;
	    while($flag){
		$l = $theslines[$linenum++];
		@temparr = split(/\|/,$l);
		if($temparr[0] eq $key){
		    $temparr[3] =~ s/<SUBPARA HEAD> //;
		    push @output, $temparr[3];
		    $flag=1;
		}else{
		    $flag=0;
		}
	    }
	    $thesoutput{$key}=[ @output ];
	    $#output=-1;
	}
      }
    return %thesoutput;
}

sub getWords{
    my $class=shift;
    my $expression=shift;
    $expression =~ s/\s//g; #remove all spaces in expression
    my @searchwords = split(/,/,$expression); #separate all search terms 
    my $linenums;
    my @slines;
    my %slines;
    my %interhash;
    my @intersection;
    my %output=();

    my $rec;
    my $chunk;
    my @chunks;

    foreach $word (@searchwords){
	$linenums=$wordhash{$word};
	@slines = split(/:/,$linenums);
	foreach $sword (@slines){#remove duplicates
	    $slines{$sword}=1;
	}
	foreach $sword (keys %slines){
	    $interhash{$sword}++;
	}
	foreach $key(keys %slines){
	    delete $slines{$key};
	}
    }

    foreach $key (keys %interhash){
	if($interhash{$key}==($#searchwords+1)){
	    push @intersection, $key;
	}
    }

    if($#intersection>=0){
	foreach $linenum (@intersection){
	    $rec=$dictlines[$linenum-1];

	    #store head word
	    $rec =~ m/<HEAD>(.*?)<\/HEAD>/;
	    if(defined $1){
		$hwd=$1;
	    }
	    #store chunks in an array
	    do{
		if($rec =~ m/<CHUNK>(.*?)<\/CHUNK>/){
		    $rec=$'; #'
		    if(defined $1){
			$chunk = $1;
			push @chunks, $chunk;
		    }else{
			$chunk="";
		    }
		}else{
		    $chunk="";
		}
	    }while($chunk);

	    #repeat for each chunk
	    foreach $chunk (@chunks){
		#find pos for chunk
		$chunk =~ m/<POS>(.*?)<\/POS>/;
		if(defined $1){
		    $currentpos = $1;
		}

		#find definitions
		do{
		    if($chunk =~ m/<DEF .*?>(.*?)<\/DEF>/){
			$chunk=$'; #'
			$thesnums="";
			if(defined $1){
			    $def=$1;
			    #find all the thes entries for the given definition
			    $temp=$def;
			    do{
				if($temp =~ m/<THES>(.*?)<\/THES>/){
				    $temp = $'; #'
				    if(defined $1){
					$thes = $1;
					if($thesnums){
					    $thesnums .=",$thes";
					}else{
					    $thesnums = $thes;
					}
				    }else{
					$thes = "";
				    }
				}else{
				    $thes="";
				}
			    }while($thes);

			    #find if words match in definitions
			    $temp = $def;
			    do{
				if($temp =~ m/<DTEXT>(.*?)<\/DTEXT>/){
				    $temp=$'; #'
				    if(defined $1){
					$dtext=$1;

					$flag=1;
					foreach $sword (@searchwords){
					    unless($dtext =~ m/\b$sword\b/i){
						$flag=0;
						last;
					    }
					}
					if($flag){
					    my $str="$hwd;$currentpos;$thesnums";
					    $output{$str}=$dtext;
					}
				    }else{
					$dtext="";
				    }
				}else{
				    $dtext="";
				}
			    }while($dtext);
			}else{
			    $def="";
			}
		    }else{
			$def="";
		    }
		}while($def);
	    }
	}
    }
    return %output;
}
# gets words, pos, thes nums of definitions which contain the given words
sub getWordsTry{
    my $class=shift;
    my $expression=shift;
    my $rec;
    my @chunks; #array of all chunks
    my $currentpos;
    my $def;
    my $thesnums;
    my %output;
    my $dtext;
    my $temp;
    my $sword;

    $expression =~ s/\s//g; #remove all spaces in expression
    my @searchwords = split(/,/,$expression); #separate all search terms 

    foreach $word ( keys %linenumhash){
	$rec = query($word);
	do{
	    if($rec =~ m/<CHUNK>(.*?)<\/CHUNK>/){
		$rec=$'; #'
		if(defined $1){
		    $chunk = $1;
		    push @chunks, $chunk;
		}else{
		    $chunk="";
		}
	    }else{
		$chunk="";
	    }
	}while($chunk);
	foreach $chunk (@chunks){
	    $chunk =~ m/<POS>(.*?)<\/POS>/;
	    if(defined $1){
		$currentpos = $1;
	    }
	    do{
		if($chunk =~ m/<DEF .*?>(.*?)<\/DEF>/){
		    $chunk=$'; #'
		    $thesnums="";
		    if(defined $1){
			$def=$1;
			#find all the thes entries for the given definition
			$temp=$def;
			do{
			    if($temp =~ m/<THES>(.*?)<\/THES>/){
				$temp = $'; #'
				if(defined $1){
				    $thes = $1;
				    if($thesnums){
					$thesnums .=",$thes";
				    }else{
					$thesnums = $thes;
				    }
				}else{
				    $thes = "";
				}
			    }else{
				$thes="";
			    }
			}while($thes);
			#find if words match in definitions
			$temp = $def;
			do{
			    if($temp =~ m/<DTEXT>(.*?)<\/DTEXT>/){
				$temp=$'; #'
				if(defined $1){
				    $dtext=$1;
				    $flag=1;
				    foreach $sword (@searchwords){
					unless($dtext =~ m/\b$sword\b/i){
					    $flag=0;
					    last;
					}
				    }
				    if($flag){
					my $str="$word;$currentpos;$thesnums";
					$output{$str}=$dtext;
				    }
				}else{
				    $dtext="";
				}
			    }else{
				$dtext="";
			    }
			}while($dtext);
		    }else{
			$def="";
		    }
		}else{
		    $def="";
		}
	    }while($def);
	}
    }
    return %output;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

Minas::BigMac v1.0 - Direct perl interface for the machine readable 
Macquarie dictionary.

=head1 SYNOPSIS
  
  # use the package
  use Minas::BigMac;

  # instantiate the class
  my $bm = Minas::BigMac->new;

  # set locations for the dictionary and thesaurus
  $bm->setDictionaryLocation("/home/cs/tpederse/CS8761/BigMac/bigmac.corrected2.sgml");
  $bm->setThesaurusLocation("/home/cs/tpederse/CS8761/BigMac/thes.data");

  # select a particular word
  $word="abandon";

  # code for getting definitions
  @defs = $bm->getDefinitions($word);
  print "\nDefinitions...\n";
  foreach $def (@defs){
      print "\t".++$index.". $def\n";
  }

  # code for getting pronunciations
  $index=0;
  %prons = $bm->getPronunciations($word);
  print "\nPronunciations...\n";
  foreach $prn (keys %prons){
      print "\t".++$index.". $prn\n";
  }

  # code fpr getting idiomatic phrases
  $index=0;
  @ips = $bm->getIdiomaticPhrases($word);
  print "\nIdiomatic Phrases...\n";
  foreach $ip (@ips){
      print "\t".++$index.". $ip\n";
  }

  # code for getting etymology
  $index=0;
  # word and regular expression for language for which etymology is to be found
  %etys = $bm->getEtymology($word,".*"); 
  print "\nEtymology...\n";
  foreach $ety (keys %etys){
      print "\t".++$index.". $ety -> $etys{$ety}\n";
  }

  # code for getting synonyms
  $index=0;
  %syns = $bm->getSynonyms($word);
  print "\nSynonyms...\n";
  foreach $syn (keys %syns){
    print "$syn : \n";
      foreach $word (@{$syns{$syn}}){
             print "\t".++$index.". $word\n";
      }
  }

  #code for getting words whose definition contains the given list of words
   $expr="large,house";
   %defs = $bm->getWords($expr);
   print "\nDefinitions containing $expr...\n";
   print "Format of output=> word; pos; thes : definition\n";
   $index=0;
   foreach $key (keys %defs){
	 print ++index.". $key : $defs{$key}\n";
   }


=head1 ABSTRACT

The Minas::BigMac package is used for querying the machine readable
MacQuarie dictionary.

It provides for the following features:

=over 4

=item * Get all definitions of a specified word.

=item * Get all pronunciations of a specified word.

=item * Get all idiomatic phrases of a specified word.

=item * Get the etymology of a specified word for all or a specific language origin.

=item * Get synonyms of a specified word.

=item * Get words whose definitions contain the specified list of words.

=back

=head1 DESCRIPTION

This module is a direct perl interface to the machine readable Macquarie 
dictionary. It expects a line corrected sgml (bigmac.corrected2.sgml) of the 
Macquarie dictionary and a corresponding thesaurus file (thes.data). It indexes
 both the files on first use, so it is quite slow on the first time. It assumes
 the current working directory to be writable by the user otherwise it would 
generate an error, since the index files are created in the current working 
directory.

The startup time is considerably high, but it is compensated by fast results 
for the queries.

=head2 EXPORT

[1]
function name: new 

arguments: none

return: class object of BigMac module

description: instantiates class

[2]
function name: setDictionaryLocation

arguments: location of corrected bigmac file (correctedbigmac.sgml)

return: none

description: sets location for the dictionary file, indexes it and reads the 
whole file into an array

[3]
function name: setThesaurusLocation

arguments: location of thesaurus file (thes.data)

return: none

description: sets location for the thesaurus file, indexes it and reads the 
whole file into an array

[4]
function name: getPronunciations

arguments: word

return: hash of pronunciations

description: gets the pronunciations for a given word

[5]
function name: getDefinitions

arguments: word

return: array of definitions

description: gets definitions for a given word

[6]
function name: getIdiomaticPhrases

arguments: word

return: array of idiomatic phrases

description: gets idiomatic phrases for a given word

[7]
function name: getEtymology

arguments: word, regular expression for language

return: hash of etymologies

description: returns etymology for a given word. the regular expression is used
to further filter the etymology based on the language of origin. For example,
".*" is used for matching any language. "English" can be used to match any 
language of origin which contains the word english.

[8]
function name: getSynonyms

arguments: word

return: hash of arrays whose keys are thesaurus entries and values contain an
array containing words which share the same thesaurus entries.

description: returns an hash of array of synonyms

[9]
function name: getWords

arguments: comma separated list of words to be searched in the definitions

return: hash whose keys are combination of word, part of speech and thesaurus 
entry and values contain the definition.

description: returns words whose definitions contain the list of words.


=head1 SEE ALSO

man NDBM_File

=head1 AUTHOR

Bridget Thomson McInnes, E<lt>bthomson@d.umn.eduE<gt>
Deodatta Y Bhoite, E<lt>bhoi0001@d.umn.eduE<gt>
Kailash Aurangabadkar, E<lt>aura0011@d.umn.eduE<gt>
Nitin Agarwal, E<lt>agar0054@d.umn.eduE<gt>
Yanhua Li, E<lt>lixx0333@d.umn.eduE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2002 by Bridget Thomson McInnes, Deodatta Y Bhoite, Kailash Aurangabadkar, 
Nitin Agarwal, Yanhua Li.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
