#!/usr/local/bin/perl

########################################################################
# PROGRAM: Minas::LDOCE
#
# Copyright (C), 2002
# Bridget McInnes,       bthomson@d.umn.edu
# Deodatta Y Bhoite,     bhoi0001@d.umn.edu
# Kailash Aurangabadkar, aura0011@d.umn.edu
# Nitin Agarwal,         agar0054@d.umn.edu
# Yanhua Li,             lixx0333@d.umn.edu
#
# University of Minnesota, Duluth
#
# USAGE:
#          %perl interface.pl
#
# The program allows the user to access the machine readable 
# dictionary in an easy menu driven way. This program also
# gives examples on how to use the Minas::LDOCE module.
#
#######################################################################

use LDOCE;

$ldoce = Minas::LDOCE->new("/home/cs5/bthomson/CS8761/LDOCE3/A-Z/sgml/");

print "\n\nPlease choose an option : \n";
print "8 -> Check if a word is an adjective. \n";
print "7 -> Retrieve the active code(s) of a word. \n";
print "6 -> Semi complete 'definition' of a word. \n"; 
print "5 -> Retrieve possible collocations of a word. \n";
print "4 -> Retrieve different subject codes of a word. \n";
print "3 -> Retrieve definitions+pos of a word. \n";
print "2 -> Retrieve definitions that contain a specified word(s). \n";
print "1 -> Return words with the same part of speech, subject \n";
print "     code, or active code as query word \n"; 
print "0 -> exit \n";

$option = <STDIN>;
chomp $option;

while($option != 0) {
    print "Would you like to output the data to a file (yes or no): ";
    $output = <STDIN>; chomp $output;
    if($output eq "y") { $output = "yes"; }
    if($output eq "yes") { 
	print "Please enter a file name: ";
	$filename = <STDIN>; chomp $filename;
	$filename = ">" . $filename;
    }
    
    if($option == 1) {
	print "Please enter a word you wish to query: ";
	my $word = <STDIN>; chomp $word;
	print "[a]ctive code, [p]art of speech, or [s]ubject code: ";
	my $code = <STDIN>; chomp $code;
	if($code eq 'a') { %temp = $ldoce->activecodes($word); }
	if($code eq 'p') { %temp = $ldoce->partofspeech($word); }
	if($code eq 's') { %temp = $ldoce->subjectcodes($word); }
	foreach $t (sort keys %temp) {
	    if($output eq "yes") {
		open(OUT, $filename);
		print OUT "$t : @{ $temp{$t} } \n\n";
		close(OUT);
	    }
	    else { print "$t :  @{ $temp{$t} } \n\n"; }
	}
    }
    if($option == 2) {
	print "Please enter the word(s) you wish to find. \n ";
	print "(words should be comma separated) : ";
	my $word = <STDIN>; chomp $word;
	my %temp = $ldoce->returndefinitions($word);
	foreach $t (sort keys %temp) { 
	    if($output eq "yes"){
		open(OUT, $filename);
		print OUT "$t \n";
		close(OUT);
	    }
	    else { print "$t \n"; }
	}
    }
    if($option == 3) {
	print "Please enter the word you wish to find. \n";
	print "( Word can be entered with a pos or not  \n";
	print "  For example, drive or drive#n ) : ";
	my $word = <STDIN>; chomp $word;
	my %temp = $ldoce->query($word, "def");
	foreach $t (sort keys %temp) {
	    if($output eq "yes") {
		open(OUT, $filename);
		print OUT "$word \n";
		print OUT " $t: \n";
		for $i (0..$#{ $temp{$t} } ) {
		    print OUT "   $count => $temp{$t}[$i] \n";
		} 
		close(OUT);
	    }
	    else  {
		print "$word \n";
		print " $t: \n";
		for $i (0..$#{ $temp{$t} } ) {
		    print "   $count => $temp{$t}[$i] \n";
		}
	    }
	}
    }
    if($option == 4) {
	print "Please enter word that you wish to find: ";
	my $word = <STDIN>; chomp $word;
	print "\nWhich subject code would you like? \n";
	print "( [f]irst degree, [p]arent, [l]ead): ";
	my $code = <STDIN>; chomp $code;
	my %temp = $ldoce->getsubjectcode($word, $code);
	if($output eq "yes") {
	    open(OUT, $filename);
	    print OUT "$word: \n";
	    foreach $t (sort keys %temp) { print OUT "  => $t \n"; }
	    close(OUT);
	}
	else {
	    print "$word: \n";
	    foreach $t (sort keys %temp) { print "  => $t \n"; }
	}
    }
    if($option == 5) {
	print "Please enter word that you wish to find: ";
	my $word = <STDIN>; chomp $word;
	my %temp = $ldoce->getcollocations($word);
	if($output eq "yes") {
	    open(OUT, $filename);
	    print OUT "$word: \n";
	    foreach $t (sort keys %temp) { print OUT "  => $t \n"; }
	    close(OUT);
	}
	else {
	    print "$word: \n";
	    foreach $t (sort keys %temp) { print "  => $t \n"; }
	}
    }
    
    if($option == 6) {
	print "Please enter word that you wish to find. \n";
	print "( Word can be entered with a pos or not  \n";
	print "  For example, drive or drive#n ) : ";
	my $word = <STDIN>; chomp $word;
	my %spell = $ldoce->query($word, "spell");
	my %def = $ldoce->query($word, "def");
	my %ex = $ldoce->query($word, "ex");
	my %lexunit = $ldoce->query($word, "lexunit");
	if($output eq "yes") {
	    open(OUT, $filename);
	    print OUT "\n$word : \n";
	    print OUT "\n Spelling : \n";
	    foreach $s (sort keys %spell) {
		for $i (0..$#{ $spell{$s} } ) {
		    print OUT "   $s => $spell{$s}[$i] \n";
		}
	    }	
	    print OUT "\n Lexical Unit : \n";
	    foreach $l (sort keys %lexunit) {
		for $i (0..$#{ $lexunit{$l} } ) {
		    print OUT "   $s => $lexunit{$l}[$i] \n";
		}
	    }
	    print OUT "\n Definitions: \n";
	    foreach $d (sort keys %def) {
		for $i (0..$#{ $def{$d} } ) {
		    print OUT "   $d => $def{$d}[$i] \n";
		}
	    }
	    print OUT "\n Examples: \n";
	    foreach $e (sort keys %ex) {
		for $i (0..$#{ $ex{$e} } ) {
		    print OUT "   $e => $ex{$e}[$i] \n";
		}
	    }
	    close(OUT);
	}
	else {
	    print "\n$word : \n";
	    print "\n Spelling : \n";
	    foreach $s (sort keys %spell) {
		for $i (0..$#{ $spell{$s} } ) {
		    print "   $s => $spell{$s}[$i] \n";
		}
	    }	
	    print "\n Lexical Unit : \n";
	    foreach $l (sort keys %lexunit) {
		for $i (0..$#{ $lexunit{$l} } ) {
		    print "   $s => $lexunit{$l}[$i] \n";
		}
	    }
	    print "\n Definitions: \n";
	    foreach $d (sort keys %def) {
		for $i (0..$#{ $def{$d} } ) {
		    print "   $d => $def{$d}[$i] \n";
		}
	    }
	    print "\n Examples: \n";
	    foreach $e (sort keys %ex) {
		for $i (0..$#{ $ex{$e} } ) {
		    print "   $e => $ex{$e}[$i] \n";
		}
	    }
	    
	}
    }
    if($option == 7) {
	print "Please enter word that you wish to find: ";
	my $word = <STDIN>; chomp $word;
	my %temp = $ldoce->getactivecode($word);
	if($output eq "yes") {
	    open(OUT, $filename);
	    print OUT "$word: \n";
	foreach $t (sort keys %temp) { print OUT "  => $t \n"; }
	}
	else {
	    print "$word: \n";
	    foreach $t (sort keys %temp) { print "  => $t \n"; }
	}
    }
    
    if($option == 8) {
	print "Please enter word that you wish to check: ";
	my $word = <STDIN>; chomp $word;
	my $temp = $ldoce->isadjective($word);
	if($output eq "yes") {
	    open(OUT, $filename);
	    print OUT "$word => ";
	    ($temp == 0) ? (print OUT " NO\n") : (print OUT " YES \n");
	}
	else {
	    print "$word => ";
	    ($temp == 0) ? (print " NO\n") : (print " YES \n");
	}
    }
   
    print "\n\nPlease choose an option : \n";
    print "8 -> Check if a word is an adjective. \n";
    print "7 -> Retrieve the active code(s) of a word. \n";
    print "6 -> Semi complete 'definition' of a word. \n"; 
    print "5 -> Retrieve possible collocations of a word. \n";
    print "4 -> Retrieve different subject codes of a word. \n";
    print "3 -> Retrieve definitions+pos of a word. \n";
    print "2 -> Retrieve definitions that contain a specified word(s). \n";
    print "1 -> Return words with the same part of speech, subject \n";
    print "     code, or active code as query word \n"; 
    print "0 -> exit \n";
    $option = <STDIN>;
}  
	
