########################################################################
# PACKAGE: Minas::WebReader
#
# Copyright (C), 2002
# Bridget McInnes,       bthomson@d.umn.edu
# Deodatta Y Bhoite,     bhoi0001@d.umn.edu
# Kailash Aurangabadkar, aura0011@d.umn.edu
# Nitin Agarwal,         agar0054@d.umn.edu
# Yanhua Li,             lixx0333@d.umn.edu
#
# University of Minnesota, Duluth
#
# USAGE:
#           use Minas::WebReader
#
# The package Minas::WebReader searches for the word on the Internet in 
# a unique way and returns sentences that contain that word. For more
# information please see perldoc.
#
#           perldoc Minas::WebReader
#
#######################################################################
package Minas::WebReader;

use 5.008;
use strict;

require Exporter;

our @ISA = qw(Exporter);

BEGIN {
    use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
    # List of classes from which we are inheriting methods
    @ISA = qw(Exporter);
    # Automatically loads these function names to be used without qualification
    @EXPORT = qw();
    # Allows these functions to be used without qualification
    @EXPORT_OK = qw();
    $VERSION = 1.0;
}


# Preloaded methods go here.

sub new
{
    # First argument is class
    my $class = shift;

    my $self = {};
    bless $self, $class;
    $self->{dir} = shift if (defined(@_ > 0));
    $self->{verbose} = @_ ? shift : 0;
    warn "Dir = ", $self->{dir}, "\n" if ($self->{verbose});
    warn "Verbose = ", $self->{verbose}, "\n" if ($self->{verbose});
    setquerysites();    
    return $self;
}
 
#set sites wish to query
my @querysites;
sub setquerysites() 
{   
    @querysites = ();            #initialize array
    open(SITES, "sites.config"); #open sites file
    while(<SITES>) {             #get the sites
	chomp;                   #chomp
	push @querysites, $_;    #put on sites array
    }
    close(SITES);
}

# finds sentences for given words from the web
sub query
{
    my $self = shift;
    my $word = shift;
    my $num = shift;

    if($num == 0) { $num = 100000; }

    use URI::Escape;
    use LWP;

    my @words = split/\,/, $word;    #comma separated
    
    #set query information
    my $query1 = 'http://www.altavista.com/web/results?pg=aq&avkw=qtrp&aqmode=s&aqa=';
    my $query2 = '&aqp=&aqo=&aqn=&aqb=&aqs=&kgs=0&kls=1&dt=tmperiod&d2=0&d0=&d1=&filetype=&rc=dmn&swd=';
    my $query3 = '&lh=&sc=on&nbq=150';
    my $query4 = '&stq=';
    
    my @pages = ('000', '100', '200', '300', '400');                            #set page numbers
    
    my %sentences = ();                                                         #initilize return hash
    foreach $word (@words) {                                                    #for word(s) to be queried
	my %sentencehash = ();                                                  #intialize sentence hash
	my $sentencecounter = 0;                                                #initialize sentence counter
	foreach my $qsite (@querysites) {                                       #forevery query site
	    if($sentencecounter >= $num) { last; }
	    foreach my $page(@pages) {
		if($sentencecounter >= $num) { last; } 
		my $url;                                                        #set url
		if($page eq '000') { $url = $query1 . $word . $query2 . $qsite . $query3; }
		else { $url = $query1 . $word . $query2 . $qsite . $query3 . $query4 . $page; }
		my $browser = LWP::UserAgent->new;                              #set the browser
		my $resp = $browser->get($url);                                 #get the url query
		if ($resp->is_success) { }                                      #if success do nothing
		else { print $resp->status_line, "$url \n"; }                   #if not print error  
		
		my $webpage = $resp->content;                                   #get webpage of the query
		
		my %sites = ();                                                 #initialize sites hash
		#get the different urls from the web page
		while($webpage =~ m/(status\=\'http\:\/\/www1?\.(.+)\.(html?|com|org|edu|gov|net)\/?)\/\'\;/gi ) {
		    my $temp = $&;                                              #store in temp
		    $temp =~ /(www1?\.(.+)\.(html?|com|org|edu|gov|net)\/?)/;   #set the www url
		    my $page = "http://" . $&;                                  #put http:// in front of the url
		    $sites{$page}++;                                            #store url in  sites hash 
		}   
		
		my @dirty = ();                                                 #initialize dirty array
		foreach my $site (sort keys %sites) {                           #for each of the urls 
		    if($sentencecounter >= $num) { last; }
		    $resp = $browser->get($site);                               #get the web page
		    my $page = $resp->content;                                  #set the content
		    
		    #cleaning up web site
		    my @dirty = split /\r/, $page;                              #split on carriage return 
		    my @clean = ();                                             #initialize  array
		    foreach my $dirty (@dirty) {                                #foreach of the dirty
			if($sentencecounter >= $num) { last; }
			chomp $dirty;                                           #chomp dirty
			my @z = split/\+/, $word;                               #split words on plus
			my $found=1;                                            #initialize found
			for my $i (0 .. $#z){                                   #for every word queried
			    unless($dirty=~m{\s+$z[$i]\s+}gi){                  #if it doesn't exists in dirty
				$found=0;                                       #set found to false
			    } 
			} 
			if($found){                                             #if all the wds found 
			    my @temp = split /\s+/, $dirty;                     #split on space
			    foreach my $t (@temp) { chomp $t;}                  #get rid of the returns
			    my $j = join " ", @temp;                            #join it back together
			    $j=~s/<.+>//gi;                                     #get rid of the tags
			    my @possibles = split /[.!?]/, $j;                  #split on sentence boundaries
			    foreach my $p (@possibles) {                        #for every possible sentence
				$p = $p . ".";                                  #finish sentence
				my $found2=1;                                   #initialize found2
				for my $ind (0 .. $#z){                         #for every word queried
				    unless($p=~m{\s+$z[$ind]\s+}gi){            #ensure that it is in the sentence
					$found2=0;                              #if not set found2 to false
				}
				}			    
				if($found2){                                    #if found2 is true
				    my $len = length $p;
				    if($len > 70) {                             #if length is > 70 store
					$sentencehash{$p}++;                    #store in hash
					$sentencecounter++;                     #increment sentence counter
					if($sentencecounter >= $num) { last; }
				    }
				}
			    }     
			}
		    }
		} #end foreach site
	    } #end foreach page
	} #end foreach qsite	
	my @sentencearray = keys %sentencehash; 
	$sentences{$word} = [ @sentencearray ];                                 #store array in hash
    } # end foreach word
    return %sentences;
}

sub getquerynumber
{
    #code idea found in Perl & LWP on page 26
    my $self = shift;
    my $word = shift;

    use URI::Escape;
    use LWP;   

    my $query = 'http://www.altavista.com/sites/search/web?q=%22' . $word . '%22&kl=XX';
    my $browser = LWP::UserAgent->new;
    my $response=$browser->get($query);
    
    my $webpage = $response->content;	
    if ($response->is_success) { }   #if success do nothing
    else { print $response->status_line, "$query \n"; }              

    my $num = 0;
    if($webpage =~m/>AltaVista found (.+) results/ig) {
	$num = $1;
    }
        
   
    return $num;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is stub documentation for your module. You'd better edit it!


=head1 NAME

Minas::WebReader - Perl extension for finding sentences on the internet 
containing specified words.

=head1 SYNOPSIS

#

# Use the module 

# 

  use Minas::WebReader;
#
# instantiate the class
#
  $oye = Minas::WebReader->new;
#
# get comma separated words from user
#
  print "Enter list of comma separated words (something,like,this): ";
  $words = <STDIN>;
#
# query for words and get hash of arrays as the output
# with words as the keys and arrays containing sentences as their
# values
#
  %sentences = $oye->query($words);
#

# write code for displaying those sentences
#
  foreach $key (keys %sentences){
      print "Sentences found for the word $key:\n\n";
      @sentencearray = $sentences{$key};
      foreach $sentence (@sentencearray){
           print "\t$sentence\n";
      }
  }
#
# end of code
#

#
# get number of documents returned by query
#
#  print "Enter word: ";
#  $word = <STDIN>;

#  $count = $oyster->getquerynumber($word);
#
#  query for words and return a number.
#
#

=head1 ABSTRACT

The WebReader module is used for searching for sentences on the internet 
containing specified words. It uses Altavista as the search engine and
searches for the given word in pre-specified sites.

=head1 DESCRIPTION

The WebReader module query function searches for the word on the internet in 
a unique way. It uses a combination of both search engine and a spider to 
find sentences containing the specified words. The disadvantage on a search 
engine is that it returns sites containing not so `good' sentences. While the 
drawback of a spider is that it may not find the word in the site it is searching. 
Hence, we decided to use a hybrid approach towards finding those sentences by using
the search engine options to search a specific website for that word. We found
that the sentences we get this way are better either using only the search 
engine strategy or the spider strategy.

We identified a list of websites addressing different subject codes to search 
for the words, so that we get sentences for most of the words. We also ensure
that we get different glosses of the word in this way.

=over 4

=item * General       http://www.cnn.com

=item * Medicine      http://www.nlm.nih.gov

=item * News          http://www.usnews.com

=item * Business      http://www.businessweek.com

=item * Philosophy    http://www.utm.edu/research/iep

=item * History       http://www.historychannel.com

=item * Wildlife      http://www.fauna-flora.org

=item * Crime         http://www.crimelibrary.com

=item * Energy        http://eia.doe.gov

=item * Religion      http://www.religion-online.org

=item * Aquatic       http://www.aquanet.com

=back

The websites are read from the file "sites.config". This allows the user to 
edit the websites if desired.

The input for the WebReader module is a list of comma separated words. By
default the module assumes the words to be ORed. The module will return
a list of sentences for each word. Search queries can also be made on the
boolean AND. This is done by separating the word with a '+' for example,  
abandon+world which will return the sentences containing both abandon 
AND world. Please note that the queries may take some time and the 
queries return may not be very good due to the pages given to the module
by altavista. The list of word to be queried can be a combination of AND
word and ORed words, e.g. save+earth,kennedy,computer+graphics.

The getquerynumber function returns the number of documents found containing
a specified word.

Use the interface.pl script to run the interface to this module.

=head2 EXPORTED METHODS

=over 4

=item * Minas::WebReader->new;

arguments : none

returns : object of class WebReader

description : constructor for the WebReader class

=item * $oyster->query($word,$num);

arguments : ($word) comma separated words to query the internet for sentences 
containing those words

arguments : ($num) number of sentences to return for the word. 

returns : hash of arrays containing words as the keys and array of sentences
for each of those keys as values

description : searches the internet for sentences containing the specified list
of words

=item * $oyster->getquerynumber($word);

arguments : ($word) one word or words separated by the '+' sign for a boolean 
AND search

returns   : the number of documents alta vista found for the specified query
word.

=back

=head1 SEE ALSO

man LWP, perldoc LWP, perldoc LWP::UserAgent

=head1 AUTHOR

In alphabetical sequence:

=over 4

=item * Bridget McInnes, E<lt>bthomson@d.umn.eduE<gt>

=item * Deodatta Y Bhoite, E<lt>bhoi0001@d.umn.eduE<gt>

=item * Kailash Aurangabadkar, E<lt>aura0011@d.umn.eduE<gt>

=item * Nitin Agarwal, E<lt>agar0054@d.umn.eduE<gt>

=item * Yanhua Li, E<lt>lixx0333@d.umn.eduE<gt>

=back

=head1 COPYRIGHT AND LICENSE

Copyright 2002 by Bridget McInnes, Deodatta Y Bhoite, Kailash Aurangabadkar,
Nitin Agarwal, Yanhua Li.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
